import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:get/get.dart';
import 'package:path_provider/path_provider.dart';
import 'package:rdc_flutter_sdk/src/controllers/device_info_controller.dart';
import 'package:rdc_flutter_sdk/src/controllers/location_controller.dart';
import 'package:rdc_flutter_sdk/src/controllers/rabbit_config_controller.dart';
import 'package:rdc_flutter_sdk/src/controllers/user_info_controller.dart';
import 'package:rdc_flutter_sdk/src/controllers/version_controller.dart';
import 'package:rdc_flutter_sdk/src/model/rabbit_enum.dart';
import 'package:rdc_flutter_sdk/src/model/rabbit_request.dart';
import 'package:rdc_flutter_sdk/src/repositories/event_repo.dart';
import 'package:uuid/uuid.dart';

import '../util/date_time_helper.dart';

class EventController {
  /* ------------ repo ------------*/
  final _eventRepo = EventRepo();
  /* ------------ repo ------------*/

  /* ------------ controller ------------*/
  final _rabbitConfigController = Get.find<RabbitConfigController>();
  final _deviceInfoController = Get.find<DeviceInfoController>();
  final _userInfoController = Get.find<UserInfoController>();
  final _locationController = Get.find<LocationController>();
  final _versionController = Get.find<VersionController>();
  /* ------------ controller ------------*/

  /* ------------ variable ------------*/
  final RabbitRequest rabbitRequest = RabbitRequest();
  var _session = Session.empty;
  var _sessionId = '';
  var isLoading = false.obs;
  /* ------------ variable ------------*/

  EventController init() {
    _initRequest();
    //_clearData();
    return this;
  }

  _initRequest() {
    rabbitRequest
      ..sdkVersion = _versionController.sdkVersion
      ..globalId = _deviceInfoController.globalId
      ..osType = _deviceInfoController.osType
      ..osVer = _deviceInfoController.osVersion
      ..display = _deviceInfoController.display
      ..maid = _deviceInfoController.maId
      ..device = _deviceInfoController.device
      ..model = _deviceInfoController.model
      ..brandName = _deviceInfoController.brandName
      ..board = _deviceInfoController.board;
  }

  Future<void> sessionStart() async {
    _session = Session.start;
    _sessionId = const Uuid().v4();
    await addEvent(
      eventName: 'session_start',
      eventValue: 'session_start',
    );
  }

  Future<void> sessionEnd() async {
    _session = Session.end;
    await addEvent(eventName: 'session_end', eventValue: 'session_end');
    _session = Session.empty;
    _sessionId = '';
    await _exportJson();
  }

  Future<void> _clearData() async {
    await _eventRepo.deleteAll();
  }

  Future<void> addEvent(
      {required String eventName,
      required String eventValue,
      Map<dynamic, dynamic>? dataLayer}) async {
    // refresh value
    _locationController.getPosition();

    var data = rabbitRequest
      ..memberId = _userInfoController.userInfo.memberId
      ..ssoId = _userInfoController.userInfo.ssoId
      ..sessionId = _sessionId.isEmpty ? const Uuid().v4() : _sessionId
      ..sessionEvent = _session
      ..sdkVersion = _versionController.sdkVersion
      ..eventName = eventName
      ..eventValue = eventValue
      ..osVer = _deviceInfoController.osVersion
      ..localeCountry = _locationController.localeCountry
      ..localeLang = _locationController.localeLang
      ..latitude = _locationController.position.value == null
          ? '0.0'
          : _locationController.position.value!.latitude.toString()
      ..longitude = _locationController.position.value == null
          ? '0.0'
          : _locationController.position.value!.longitude.toString()
      ..appVer = _versionController.appVer
      ..appVerNum = _versionController.appVerNum
      ..prevAppVer = _versionController.prevAppVer
      ..prevAppVerNum = _versionController.prevAppVerNum
      ..dataLayer = dataLayer
      ..time = DateTime.now().format('yyyy-MM-dd:HH:mm:ss');

    if (_rabbitConfigController.rabbitDataConfig.dbType == DBType.realm) {
      var lastId = await _eventRepo.lastId();
      data.id = (lastId ?? 0) + 1;
    }

    await _eventRepo.insert(data);
  }

  Future<void> uploadEvent() async {
    var requests = await _eventRepo.getRequests();
    if (requests.isEmpty) return;
    await _eventRepo.updateSend(requests.map((e) => e.id).toList());
  }

  Future<void> clearCache() async {
    var data = await _eventRepo.getAll();
    var lastId = await _eventRepo.lastId();
    await _eventRepo.deleteAllSent();
  }

  _exportJson() async {
    try {
      var results = await _eventRepo.getAll();
      var appDocDir = await getApplicationDocumentsDirectory();

      final File file = File('${appDocDir.path}/rabbit_data.json');
      await file.writeAsString(json.encode(results));
    } catch (e) {
      print(e.toString());
    }
  }
}
