import 'dart:convert';
import 'dart:io';

import 'package:get/get.dart';
import 'package:hive/hive.dart';
import 'package:path_provider/path_provider.dart';
import 'package:rdc_flutter_sdk/src/controllers/rabbit_config_controller.dart';
import 'package:rdc_flutter_sdk/src/controllers/user_info_controller.dart';
import 'package:rdc_flutter_sdk/src/database/db_interface.dart';
import 'package:rdc_flutter_sdk/src/database/realm_db.dart';
import 'package:rdc_flutter_sdk/src/model/rabbit_enum.dart';
import 'package:rdc_flutter_sdk/src/model/rabbit_data_config.dart';
import 'package:rdc_flutter_sdk/src/model/rabbit_request.dart';
import 'package:rdc_flutter_sdk/src/model/realm/rabbit_request_realm.dart';
import 'package:rdc_flutter_sdk/src/model/realm/user_info_realm.dart';
import 'package:rdc_flutter_sdk/src/model/realm/version_realm.dart';
import 'package:rdc_flutter_sdk/src/rabbit_data.dart';
import 'package:realm/realm.dart';

import 'controllers/condition_controller.dart';
import 'controllers/device_info_controller.dart';
import 'controllers/event_controller.dart';
import 'controllers/location_controller.dart';
import 'controllers/version_controller.dart';
import 'database/hive_db.dart';
import 'model/hive/rabbit_request_hive.dart';
import 'model/hive/user_info_hive.dart';
import 'model/hive/version_hive.dart';
import 'model/hive_config.dart';
import 'model/user_info.dart';

class RabbitDataImpl implements RabbitDataInstance {
  UserInfoController get _userInfoController {
    UserInfoController controller = Get.find();
    return controller;
  }

  RabbitConfigController get _rabbitConfigController {
    RabbitConfigController controller = Get.find();
    return controller;
  }

  EventController get _eventController {
    EventController controller = Get.find();
    return controller;
  }

  @override
  Future<RabbitDataInstance> initializeInstance(RabbitDataConfig config) async {
    await Get.putAsync(() async => await RabbitConfigController().init(config));
    await _initDB();
    // init controller
    await Get.putAsync(() async => await VersionController().init());
    await Get.putAsync(() async => await ConditionController().init());
    await Get.putAsync(() async => await DeviceInfoController().init());
    await Get.putAsync(() async => await LocationController().init());
    await Get.putAsync(() async => await UserInfoController().init());
    Get.put(EventController().init());

    return RabbitDataImpl();
  }

  /* ------------------- override config ------------------- */
  @override
  void setAPIKey(String key) {
    _rabbitConfigController.rabbitDataConfig.apiKey = key;
  }

  @override
  void setEncryptionKey(String key) {
    _rabbitConfigController.rabbitDataConfig.encryptKey = key;
  }

  @override
  void setDataCollectionType(DataCollectionType type) {
    _rabbitConfigController.rabbitDataConfig.collectionType = type;
  }

  @override
  void setDeleteAfterSent(bool value) {
    _rabbitConfigController.rabbitDataConfig.deleteAfterSent = value;
  }

  @override
  void setRetentionDays(int date) {
    _rabbitConfigController.rabbitDataConfig.retentionDays = date;
  }

  @override
  void setRetryOnFailed(int time) {
    _rabbitConfigController.rabbitDataConfig.retryTime = time;
  }

  @override
  void setRotationTime(Duration duration) {
    _rabbitConfigController.rabbitDataConfig.rotationTime = duration;
  }

  @override
  void setTriggerCondition(
      {double cpu = 0.0, double memory = 0.0, double betteryLevel = 0.0}) {
    _rabbitConfigController.rabbitDataConfig.trigger =
        TriggerCondition(cpu: cpu, memory: memory, betteryLevel: betteryLevel);
  }

  @override
  void setDatabaseType(DBType dbType) {
    _rabbitConfigController.rabbitDataConfig.dbType = dbType;
  }

  /* ------------------- override config ------------------- */

  /* ------------------- set user info  ------------------- */
  @override
  Future<void> setUserInfo(String memberId, String ssoId) async {
    _userInfoController.userInfo.memberId = memberId;
    _userInfoController.userInfo.ssoId = ssoId;
    await _userInfoController.save();
  }
  /* ------------------- set user info  ------------------- */

  /* ------------------- event method  ------------------- */
  @override
  Future<void> addEvent(
      {required String eventName,
      required String eventValue,
      String sessionEvent = '',
      Map<dynamic, dynamic>? dataLayer}) async {
    return await _eventController.addEvent(
        eventName: eventName, eventValue: eventValue, dataLayer: dataLayer);
  }

  @override
  Future<void> uploadEvent() async {
    return await _eventController.uploadEvent();
  }

  @override
  Future<void> clearCache() async {
    return await _eventController.clearCache();
  }

  @override
  Future<void> sessionStart() async {
    await _eventController.sessionStart();
  }

  @override
  Future<void> sessionEnd() async {
    await _eventController.sessionEnd();
  }

  /* ------------------- event method  ------------------- */

  // inital database provider
  Future<void> _initDB() async {
    if (_rabbitConfigController.rabbitDataConfig.dbType == DBType.hive) {
      // register adapter
      Hive.registerAdapter(RabbitRequestHiveAdapter());
      Hive.registerAdapter(VersionHiveAdapter());
      Hive.registerAdapter(UserInfoHiveAdapter());

      await Get.putAsync<HiveDBInterface>(() async => await _initHive(),
          tag: 'dbHive');
    } else {
      await Get.putAsync<RealmDBInterface>(() async => await _initRealm(),
          tag: 'dbRealm');
    }
  }

  Future<HiveDBInterface> _initHive() async {
    Directory appDocDir = await getApplicationDocumentsDirectory();
    var config = HiveConfig(
        dbName: 'rdc_flutter_sdk',
        path: appDocDir.path,
        boxNames: {UserInfo.boxName, RabbitRequest.boxName},
        cipher: _rabbitConfigController.rabbitDataConfig.encryptKey.isEmpty
            ? null
            : HiveAesCipher(base64Url
                .decode(_rabbitConfigController.rabbitDataConfig.encryptKey)));
    return HiveDBImpl(config);
  }

  Future<RealmDBInterface> _initRealm() async {
    Directory appDocDir = await getApplicationDocumentsDirectory();
    final realmDirectory = '${appDocDir.path}/mongodb-realm';
    if (!Directory(realmDirectory).existsSync()) {
      await Directory(realmDirectory).create(recursive: true);
    }

    var config = Configuration.local(
        [RabbitRequestRealm.schema, VersionRealm.schema, UserInfoRealm.schema],
        path: '$realmDirectory/rdc_flutter_sdk.realm');
    var realm = Realm(config);
    return RealmDBImpl(realm);
  }
}
