import 'package:get/get.dart';
import 'package:rdc_flutter_sdk/src/controllers/rabbit_config_controller.dart';
import 'package:rdc_flutter_sdk/src/database/db_interface.dart';
import 'package:rdc_flutter_sdk/src/model/base_model.dart';
import 'package:rdc_flutter_sdk/src/model/rabbit_enum.dart';

// T generic class, H hive object, R realm object
class BaseRepo<T extends BaseModel, H extends HiveBaseModel<T>,
    R extends RealmBaseModel<T>> {
  final config = Get.find<RabbitConfigController>();
  late HiveDBInterface hiveDB;
  late RealmDBInterface realmDB;
  String boxName = '';

  BaseRepo(this.boxName) {
    if (config.rabbitDataConfig.dbType == DBType.hive) {
      hiveDB = Get.find<HiveDBInterface>(tag: 'dbHive');
    } else {
      realmDB = Get.find<RealmDBInterface>(tag: 'dbRealm');
    }
  }

  Future<List<T>> getAll() async {
    if (config.rabbitDataConfig.dbType == DBType.hive) {
      var db = await hiveDB.getAll<H>(boxName);
      List<T> results = [];
      for (var e in db.entries) {
        results.add(e.value.mapFromHive(e.key));
      }
      return results;
    }

    var results = realmDB.getAll<R>();

    return results.map((e) => e.mapFromRealm()).toList();
  }

  Future<T?> get(int id) async {
    try {
      if (config.rabbitDataConfig.dbType == DBType.hive) {
        var db = await hiveDB.get<H>(boxName, id);
        return db.isEmpty ? null : db[id]?.mapFromHive(id);
      }

      var result = realmDB.get<R>(id);
      return result?.mapFromRealm();
    } catch (e) {
      print(e.toString());
    }
    return null;
  }

  Future<void> insert(T data) async {
    try {
      if (config.rabbitDataConfig.dbType == DBType.hive) {
        return await hiveDB.insert<H>(boxName, data.mapToHive());
      }

      realmDB.insert<R>(data.mapToRealm());
    } catch (e) {
      print(e.toString());
    }
  }

  Future<void> update(int id, T data) async {
    try {
      if (config.rabbitDataConfig.dbType == DBType.hive) {
        return await hiveDB.upsert<H>(boxName, id, data.mapToHive());
      }

      realmDB.edit<R>(id, data.mapToRealm());
    } catch (e) {
      print(e.toString());
    }
  }

  Future<void> delete(int id) async {
    try {
      if (config.rabbitDataConfig.dbType == DBType.hive) {
        await hiveDB.delete<H>(boxName, id);
        return;
      }
      var data = await get(id);
      if (data == null) return;

      realmDB.delete<R>(data.mapToRealm());
    } catch (e) {
      print(e.toString());
    }
  }

  Future<void> deleteAllKeys(List<int> ids) async {
    try {
      if (config.rabbitDataConfig.dbType == DBType.hive) {
        await hiveDB.deleteKeys<H>(boxName, ids);
        return;
      }

      for (var id in ids) {
        var data = realmDB.get<R>(id);
        if (data == null) return;
        realmDB.delete<R>(data);
      }
    } catch (e) {
      print(e.toString());
    }
  }

  Future<void> deleteAll() async {
    try {
      if (config.rabbitDataConfig.dbType == DBType.hive) {
        await hiveDB.deleteAll<H>(boxName);
        return;
      }
      realmDB.deleteAll<R>();
    } catch (e) {
      print(e.toString());
    }
  }

  Future<int?> lastId() async {
    try {
      var data = await getAll();
      if (data.isEmpty) return null;
      return data.last.id;
    } catch (e) {
      print(e.toString());
    }
    return null;
  }
}
